@@ -30,6 +30,16 @@ class Work(ModelSQL, ModelView):
             'readonly': Bool(Eval('timesheet_lines', [0])),
             },
         help="Allow to fill in timesheets with this work")
+    timesheet_start_date = fields.Date('Timesheet Start',
+        states={
+            'invisible': ~Eval('timesheet_available'),
+            },
+        depends=['timesheet_available'])
+    timesheet_end_date = fields.Date('Timesheet End',
+        states={
+            'invisible': ~Eval('timesheet_available'),
+            },
+        depends=['timesheet_available'])
     company = fields.Many2One('company.company', 'Company', required=True,
         select=True)
     timesheet_lines = fields.One2Many('timesheet.line', 'work',
@@ -43,14 +53,10 @@ class Work(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(Work, cls).__setup__()
-        cls._constraints += [
-            ('check_recursion', 'recursive_works'),
-            ('check_parent_company', 'parent_company'),
-            ]
         cls._error_messages.update({
-                'recursive_works': 'You can not create recursive works!',
-                'parent_company': 'Every work must be in the same company '\
-                    'as it\'s parent work!',
+                'invalid_parent_company': ('Every work must be in the same '
+                    'company as it\'s parent work but "%(child)s" and '
+                    '"%(parent)s" are in different companies.'),
                 })
 
     @staticmethod
@@ -73,10 +79,21 @@ class Work(ModelSQL, ModelView):
     def default_company():
         return Transaction().context.get('company')
 
+    @classmethod
+    def validate(cls, works):
+        super(Work, cls).validate(works)
+        cls.check_recursion(works, rec_name='name')
+        for work in works:
+            work.check_parent_company()
+
     def check_parent_company(self):
         if not self.parent:
-            return True
-        return self.parent.company == self.company
+            return
+        if self.parent.company != self.company:
+            self.raise_user_error('invalid_parent_company', {
+                    'child': self.rec_name,
+                    'parent': self.parent.rec_name,
+                    })
 
     @classmethod
     def _tree_qty(cls, hours_by_wt, children, ids, to_compute):
@@ -102,9 +119,9 @@ class Work(ModelSQL, ModelView):
         all_ids = [w.id for w in all_works]
         # force inactive ids to be in all_ids
         all_ids = list(set(all_ids + ids))
-        clause = "SELECT work, sum(hours) FROM timesheet_line "\
-                     "WHERE work IN (%s) "\
-                     % ",".join(('%s',) * len(all_ids))
+        clause = ("SELECT work, sum(hours) FROM timesheet_line "
+            "WHERE work IN (%s) "
+            % ",".join(('%s',) * len(all_ids)))
         date_cond = ""
         args = []
         if Transaction().context.get('from_date'):
@@ -134,6 +151,21 @@ class Work(ModelSQL, ModelView):
         else:
             return self.name
 
+    @classmethod
+    def search_rec_name(cls, name, clause):
+        if isinstance(clause[2], basestring):
+            values = clause[2].split('\\')
+            values.reverse()
+            domain = []
+            field = 'name'
+            for name in values:
+                domain.append((field, clause[1], name.strip()))
+                field = 'parent.' + field
+        else:
+            domain = [('name',) + tuple(clause[1:])]
+        ids = [w.id for w in cls.search(domain, order=[])]
+        return [('parent', 'child_of', ids)]
+
     @classmethod
     def copy(cls, works, default=None):
         if default is None:
